import re

# Secuencias ANSI
BOLD = "\033[1m"
ITALIC = "\033[3m"
RESET = "\033[0m"
CODE = "\033[38;5;28m"
BLOCK_CODE = "\033[48;5;235m\033[38;5;28m"
HEADER1 = "\033[1m\033[38;5;34m"
HEADER2 = "\033[1m\033[38;5;36m"

def process_markdown(content:str):

    content = re.sub(r'^# (.*)$', f'{HEADER1}\\1{RESET}', content, flags=re.MULTILINE)
    content = re.sub(r'^## (.*)$', f'{HEADER2}\\1{RESET}', content, flags=re.MULTILINE)
    content = re.sub(r'```(.*?)```', f'{BLOCK_CODE}\\1{RESET}', content, flags=re.DOTALL)
    content = re.sub(r'\*\*(.*?)\*\*', f'{BOLD}\\1{RESET}', content)
    content = re.sub(r'\*(.*?)\*', f'{ITALIC}\\1{RESET}', content)
    content = re.sub(r'`(.*?)`', f'{CODE}\\1{RESET}', content)

    return content


from ..core import aiResponse
from ..core.types import prompt
from ..core.aiResponse import processResponse

from ..utils import printer

from . import configOperations

def main(args: list):
    # Verifica si solo hay un parámetro (inicio)
    if len(args) == 1:
        printer.welcome()

    else:
        args.pop(0)

        PARAMETROS = {
            "name": ["-n", "--name", "-name"],
            "key": ["-k", "--key", "-key"],
            "idioma": ["-l", "--language", "-language"],
            "clear": ["-c", "-clear", "--clear"]
        }
        
        # Verifica si el primer argumento es una opción de configuración
        if args[0].startswith("-"):
            PARAM = args.pop(0)
            VALUE = args.pop(0) if len(args) > 0 else "No especificado"

            if PARAM in PARAMETROS["name"]:
                configOperations.setName(VALUE)
            elif PARAM in PARAMETROS["key"]:
                configOperations.setKey(VALUE)
            elif PARAM in PARAMETROS["idioma"]:
                configOperations.setlang(VALUE)
            elif PARAM in PARAMETROS["clear"]:
                configOperations.clear()
            else:
                print("Parámetro desconocido. Usa -h para ver las opciones disponibles.")

        # Caso en el que no se trata de configuración
        else:
            PROMPT = " ".join(args)
            userPrompt = prompt.promptUser(text=PROMPT)

            response = aiResponse.response(userPrompt)
            if response:
                for chunk in response:
                    processResponse(chunk)



def welcome():
    text_welcome = (
        "¡Bienvenido al asistente interactivo!\n\n"
        "Puedes usar este asistente para interactuar con nuestro sistema de IA.\n\n"
        "Aquí están los parámetros que puedes usar:\n"
        "  - Configuración de nombre: -n <nombre> o --name <nombre>\n"
        "  - Configuración de clave: -k <clave> o --key <clave>\n"
        "  - Configuración de idioma: -l <idioma> o --language <idioma>\n"
        "  - Limpiar memoria del asistente: -c o --clear\n\n"
        "Si solo quieres chatear, simplemente escribe tu mensaje y presiona Enter."
    )
    print(text_welcome)



import os
import json

# Ruta del archivo de configuración
CONFIG_FILE = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'config.json')

def load_config():
    """Carga la configuración desde el archivo JSON, si existe."""
    if os.path.exists(CONFIG_FILE):
        with open(CONFIG_FILE, 'r') as file:
            return json.load(file)
    return {}

def save_config(config):
    """Guarda la configuración en el archivo JSON."""
    with open(CONFIG_FILE, 'w') as file:
        json.dump(config, file, indent=4)

def ruta():
    """Devuelve la ruta absoluta del archivo actual."""
    return os.path.abspath(__file__)

def setName(name: str):
    """Configura el nombre en el archivo de configuración y lo imprime."""
    config = load_config()
    config['name'] = name
    save_config(config)
    print(f"Nombre configurado: {name}")

def setKey(key: str):
    """Configura la clave en el archivo de configuración."""
    config = load_config()
    config['key'] = key
    save_config(config)
    print(f"Clave configurada: {key}")

def setlang(lang: str):
    """Configura el idioma en el archivo de configuración."""
    config = load_config()
    config['language'] = lang
    save_config(config)
    print(f"Idioma configurado: {lang}")


from ..core.types.prompt import promptUser
from ..core.types.assistant import relu

from google.generativeai.types.generation_types import GenerateContentResponse
from ..utils.parseResponseMD import process_markdown
from ..utils.configOperations import load_config

def response(prompt:promptUser):
    key = load_config().get("key")
    if not key:
        print ("Establece una key primero, con el parámetro --key <key>, consiguelo en: https://aistudio.google.com/app/apikey")
        return None
    response = relu(data={"key": key}).generate(prompt)
    return response

def processResponse(response:GenerateContentResponse):
    strProcessed = process_markdown(response.text)
    print (strProcessed, end="", flush=True)